"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRaw = exports.whis = exports.WhoisParser = exports.TCPHelper = void 0;
const TCPHelper_1 = require("./TCPHelper");
Object.defineProperty(exports, "TCPHelper", { enumerable: true, get: function () { return TCPHelper_1.TCPHelper; } });
const WhoisParser_1 = __importDefault(require("./WhoisParser"));
exports.WhoisParser = WhoisParser_1.default;
// eslint-disable-next-line @typescript-eslint/no-var-requires
const whisData = require('whis-data');
const findWhoisServer = (tld) => tld &&
    (whisData[tld.toUpperCase()] ||
        (tld.indexOf('.') > -1 &&
            findWhoisServer(tld.substr(tld.indexOf('.') + 1))));
/**
 * Gets the raw WHOIS response from the given domain.
 *
 * @param {string} domain – the given domain of which the WHOIS query will be done
 * @param {string} [server] – custom WHOIS server to retrieve information from; if no server given, will get server from whis-data
 */
const getRaw = (domain, server) => __awaiter(void 0, void 0, void 0, function* () {
    const whoisServer = server !== null && server !== void 0 ? server : findWhoisServer(domain);
    if (!whoisServer)
        throw Error('Whois server not found');
    const tcp = new TCPHelper_1.TCPHelper(whoisServer, 43);
    return new Promise((resolve, reject) => {
        let hasError = false;
        tcp.on('error', (err) => {
            hasError = true;
            reject(err);
        });
        tcp
            .send(domain, true)
            .then((buffer) => {
            if (hasError)
                return;
            if (!buffer) {
                throw Error('Domain not found');
            }
            resolve(buffer.toString());
        })
            .catch((error) => !hasError && reject(error));
    });
});
exports.getRaw = getRaw;
/**
 * Gets parsed WHOIS data from the given domain.
 *
 * @param {string} domain – the given domain of which the WHOIS query will be done
 * @param {string} [server] – custom WHOIS server to retrieve information from; if no server given, will get server from whis-data
 */
const whis = (domain, server) => __awaiter(void 0, void 0, void 0, function* () { return (0, WhoisParser_1.default)(yield getRaw(domain, server)); });
exports.whis = whis;
exports.default = whis;
